/*
 ----------------------------------------------------------
|
| Software Name :conv2mw3 Ver 0.1 beta
|
|   FileWriter.cpp
|
|                     Written by T.Takeda,    2012/06/01
|                                Y.Sato,      2012/06/01
|
|   Contact address : IIS, The University of Tokyo CISS
|
 ----------------------------------------------------------
*/
#include "FileWriter.h"
using namespace boost::xpressive;
using namespace boost;
CFileWriter::CFileWriter()
{
	msDirichlet= "Dirichlet";
	msNeumann  = "Neumann";
}
CFileWriter::~CFileWriter()
{
}

//--
// Header && mesh version 0.4
//--
bool CFileWriter::WriteHeader(ofstream& ofs)
{
	string sVer = MW3_Tag::VerNumber();// o[Wԍ

	ofs << "//----------------------------------------      \n"
		     "//      converted mesh data                     \n"
		     "// output format  \"MW3 mesh ver " + sVer + "\" \n"
		     "//----------------------------------------      \n" << endl;
	
	//--
	// Version 
	//--
	ofs << "Version \n"      
	       "  MW3_" + sVer + " \n"
	       "End \n" 
			<< endl;

	return true;
}
//--
// AssyModel
//--
bool CFileWriter::WriteAssyModel(CAssyModel *pAssyModel, ofstream& ofs)
{
	if(!pAssyModel->existMesh()) return false;//Mesh.

	CMessage *pMsg=CMessage::Instance();
	
	/*
	bVp[c  őID@ŏID
	bVp[cID  ԍ@
	c
	ʐMe[u
	bVp[cID  NEyA1st  NEyA2nd
	c
	*/

	ofs << "AssyModel" << endl;
	//bV  MaxID  MinID
	ofs << format(" %lu %lu %lu") %pAssyModel->getNumOfMesh() %pAssyModel->getMaxIDinMesh() %pAssyModel->getMinIDinMesh() << endl;
	//MeshID  ԍ
	for(size_t i=0; i < pAssyModel->getNumOfMesh(); i++){
		CMesh* pMesh=pAssyModel->getMesh(i);
		ofs << format(" %3lu %lu") %pMesh->getID()  %0 << endl;
	};
	//ʐMe[u(VO:"0")
	ofs << format(" %3lu") %0 << endl;
	ofs << "End\n" << endl;


	//bZ[W\
	pMsg->info("wrote :1 block \"AssyModel\"");
	
	return true;
}
//--
// Node
//--
bool CFileWriter::WriteNode(CAssyModel *pAssyModel, ofstream& ofs)
{
	if(!pAssyModel->existMesh()) return false;//Mesh

	CMessage *pMsg=CMessage::Instance();

	size_t nNumMesh= pAssyModel->getNumOfMesh();

	for(size_t i=0; i < nNumMesh; i++){
		CMesh *pMesh= pAssyModel->getMesh(i);
		/*
		ߓ_  bVp[cID@őߓ_ID  ŏߓ_ID
		(V,S,SV)@XJ[ϐ̐  xNgϐ̐  ID  X  Y  Z
		c
		*/
		ofs << "Node" << endl;
		ofs << format(" %lu %lu %lu %lu") %pMesh->getNumOfNode() %pMesh->getID() %pMesh->getMaxNodeID() %pMesh->getMinNodeID() << endl;
		
		for(size_t ii=0; ii < pMesh->getNumOfNode(); ii++){
			CNode *pNode= pMesh->getNode(ii);
			ofs << format(" %s  %9lu  %23.15e %23.15e %23.15e") %"V 0 3"  %pNode->getID() %pNode->getX() %pNode->getY() %pNode->getZ() << endl;
		};
		ofs << "End\n" << endl;
	};


	// bZ[W\
	string sNumMesh= lexical_cast<string>(nNumMesh);
	pMsg->info("wrote :"+sNumMesh+" block \"Node\"");

	return true;
}
//--
// Element
//--
bool CFileWriter::WriteElement(CAssyModel *pAssyModel, ofstream& ofs)
{
	if(!pAssyModel->existMesh()) return false;//Mesh

	CMessage *pMsg=CMessage::Instance();

	// regex
	string strRegex= FistrElemTypeS::regStandard();// MW3ɑ݂vf^Cv
	sregex regBasis = sregex::compile(strRegex);
	string strRegex2= FistrElemTypeS::regShellType();// VFvf
	sregex regShell= sregex::compile(strRegex2);
	smatch res;

	size_t nNumMesh= pAssyModel->getNumOfMesh();

	for(size_t i=0; i < nNumMesh; i++){
		CMesh *pMesh= pAssyModel->getMesh(i);
		/*
		vf  bVp[cID@ővfID  ŏvfID
		(Hexa,Tetra,etc.)@ID  ߓ_ID  ߓ_ID  ߓ_ID  ߓ_ID ... ...(vfߓ_JԂ)
		c
		*/
		ofs << "Element" << endl;
		ofs << format(" %lu %lu %lu %lu") %pMesh->getNumOfElement() %pMesh->getID() %pMesh->getMaxElementID() %pMesh->getMinElementID() << endl;
		for(size_t ii=0; ii < pMesh->getNumOfElement(); ii++){
			
			CElement* pElem= pMesh->getElement(ii);
			string sType=pElem->getCType();
			size_t nType=pElem->getNType();

			//if( regex_search(sType, res, regBasis) ){
			if( regex_match(sType, regBasis)){
				//vf^Cv@ID  ߓ_ID  ߓ_ID  ߓ_ID  ߓ_ID ... ...
				//ofs << format(" %s %9lu ") %res.str()  %pElem->getID();
				ofs << format(" %s %9lu ") %sType  %pElem->getID();
				for(size_t iii=0; iii < pElem->getNumOfNode(); iii++){
					ofs << format(" %9lu")  %pElem->getNodeID_Fistr2MW3(iii);
				};
				ofs << endl;
			}else if( regex_search(sType, res, regShell) ){
				//VFvfQuad,Triangleɕϊ
				string sRepType;
				switch(nType){
				case(FistrElementType::TriShell):   sRepType=FistrElemTypeS::Triangle();   break;
				case(FistrElementType::TriShell2):  sRepType=FistrElemTypeS::Triangle2();  break;
				case(FistrElementType::QuadShell):  sRepType=FistrElemTypeS::Quad();  break;
				case(FistrElementType::QuadShell2): sRepType=FistrElemTypeS::Quad2(); break;
				default:
					pMsg->error(res.str()+" ? ");//regex΁Ȁɂ͓Ȃ.
					break;
				}
				//vf^Cv(u)@ID  ߓ_ID  ߓ_ID  ߓ_ID  ߓ_ID ... ...
				ofs << format(" %s %9lu ") %sRepType %pElem->getID();
				for(size_t iii=0; iii < pElem->getNumOfNode(); iii++){
					ofs << format(" %9lu") %pElem->getNodeID_Fistr2MW3(iii);
				};
				ofs << endl;
			}else{
				pMsg->error( res.str()+" not supported");//Error --- IFaceQuad܂.
			}
		};
		ofs << "End\n" << endl;
	};

	// bZ[W\
	string sNumMesh= lexical_cast<string>(nNumMesh);
	pMsg->info("wrote :"+sNumMesh+" block \"Element\"");

	return true;
}
//--
// BoundaryNodeMesh : ftHgDirichlet^ŏo(R:FrontISTR͋ElgpȂ)
//--
bool CFileWriter::WriteBndNodeMesh(CAssyModel *pAssyModel, ofstream& ofs)
{
	if(!pAssyModel->existMesh()) return false;//Mesh

	CMessage *pMsg=CMessage::Instance();

	size_t nNMesh=pAssyModel->getNumOfMesh();
	size_t nSumGrpN=0;
	string sBndType;

	for(size_t i=0; i < nNMesh; i++){
		CMesh *pMesh=pAssyModel->getMesh(i);
		/*
		^OFBoundaryNodeMesh
		
		bVp[cID  E
		ߓ_EbVID  Eށ@ (DOF) DOF  DOF  ... # :ver0.4ȍ~
		*/
		size_t nNNgrp=pMesh->getNumOfNgrp();
		nSumGrpN += nNNgrp;

		if(nNNgrp > 0){
			ofs << "BoundaryNodeMesh" << endl;
			ofs << format(" %lu %lu") %pMesh->getID() %nNNgrp << endl;
		
			for(size_t ii=0; ii < nNNgrp; ii++){
				CNgroup *pNgrp= pMesh->getNgrp(ii);

				switch(pNgrp->getBndType()){ //EނH
				case( BndType::Dirichlet ): sBndType = msDirichlet; break;
				case( BndType::Neumann ): sBndType = msNeumann; break;
				case( BndType::NotUse ): sBndType = msDirichlet; break;
				default: 
					sBndType = msDirichlet;
					break;
				}
			
				ofs << format(" %lu %s %s %lu") %pNgrp->getID() %sBndType %pNgrp->getGroupName() %0 << endl;//:0(f[^ɖ)
			};
			ofs << "End\n" << endl;
		}
	};

	
	for(size_t i=0; i < nNMesh; i++){
		CMesh *pMesh=pAssyModel->getMesh(i);
		/*
		^OFBoundaryNode
		( fBN&mC}  )
		ߓ_EbVID@E  bVp[cID  Eߓ_
		Eߓ_ID  ߓ_ID  X  Y  Z  Rxԍ El(FrontISTR͒lgpȂˏ0.0)
		*/
		size_t nNNgrp=pMesh->getNumOfNgrp();
		for(size_t ii=0; ii < nNNgrp; ii++){
			CNgroup *pNgrp= pMesh->getNgrp(ii);

			switch(pNgrp->getBndType()){ //EނH
			case( BndType::Dirichlet ): sBndType = msDirichlet; break;
			case( BndType::Neumann ): sBndType = msNeumann; break;
			case( BndType::NotUse ): sBndType = msDirichlet; break;
			default: 
				sBndType = msDirichlet;
				break;
			}

			ofs << "BoundaryNode" << endl;
			ofs << format(" %lu %s %lu %lu") %pNgrp->getID() %sBndType %pMesh->getID() %pNgrp->getNumOfNode() << endl;
			for(size_t iii=0; iii < pNgrp->getNumOfNode(); iii++){
				CNode* pNode=pNgrp->getNode(iii);
				ofs << format(" %9lu %9lu %23.15e %23.15e %23.15e %lu   %lf") %iii %pNode->getID() %pNode->getX() %pNode->getY() %pNode->getZ() %0 %0.0 << endl;
			};
			ofs << "End\n" << endl;
		};
	};


	// bZ[W\
	string sSumGrpN= lexical_cast<string>(nSumGrpN);
	pMsg->info("wrote :"+ sSumGrpN +" block \"BoundaryNode\"");

	return true;
}
//--
// BoundaryFaceMesh : ftHgNeumann^ŏo(R:FrontISTR͋ElgpȂ)
//--
bool CFileWriter::WriteBndFaceMesh(CAssyModel *pAssyModel, ofstream& ofs)
{
	if(!pAssyModel->existMesh()) return false;//Mesh

	CMessage *pMsg=CMessage::Instance();

	size_t nNMesh=pAssyModel->getNumOfMesh();
	size_t nSumGrpN=0;
	string sBndType;
	/*
	^OFBoundaryFaceMesh
	
	bVp[cID  E
	ʋEbVID  Eށ@  Rx Rxԍ Rxԍ ...(JԂ) (DOF) DOF  DOF  ...
	*/
	for(size_t i=0; i < nNMesh; i++){
		CMesh *pMesh=pAssyModel->getMesh(i);

		size_t nNSgrp=pMesh->getNumOfSgrp();
		nSumGrpN += nNSgrp;

		if(nNSgrp > 0){
			ofs << "BoundaryFaceMesh" << endl;
			ofs << format(" %lu %lu") %pMesh->getID() %nNSgrp << endl;
			for(size_t ii=0; ii < nNSgrp; ii++){
				CSgroup* pSgrp=pMesh->getSgrp(ii);

				switch(pSgrp->getBndType()){ //EނH
				case( BndType::Dirichlet ): sBndType = msDirichlet; break;
				case( BndType::Neumann ): sBndType = msNeumann; break;
				case( BndType::NotUse ): sBndType = msNeumann; break;
				default: 
					sBndType = msNeumann;
					break;
				}

				ofs << format(" %lu %s %s %lu %lu %lu") %pSgrp->getID() %sBndType %pSgrp->getGroupName() %1 %0 %0 << endl;//Rx:1,Rx:0,:0
			};
			ofs << "End\n" << endl;
		}
	};
	//--
	// BoundaryFace
	//--
	for(size_t i=0; i < nNMesh; i++){
		CMesh *pMesh=pAssyModel->getMesh(i);
		size_t nNSgrp=pMesh->getNumOfSgrp();
		
		for(size_t ii=0; ii < nNSgrp; ii++){
			CSgroup *pSgrp= pMesh->getSgrp(ii);

			if(pSgrp->getBndType()!=BndType::Dirichlet){
				/*
				^OFBoundaryFace
				( mC}^ )
				ʋEbVID@E  bVp[cID  Eߓ_ Face
				Eߓ_ID  ߓ_ID  
				c
				ʌ`^Cv EID  vfID  ʔԍ  Rxԍ Eߓ_ID...(JԂ)  El
				*/
				ofs << "BoundaryFace" << endl;
				ofs << format(" %lu %s %5lu %5lu %5lu") %pSgrp->getID() %msNeumann %pMesh->getID() %pSgrp->getNumOfNode() %pSgrp->getNumOfFace() << endl;
				// BNode 
				size_t nNNode=pSgrp->getNumOfNode();
				for(size_t inode=0; inode < nNNode; inode++){
					CNode* pNode = pSgrp->getNode(inode);
					ofs << format(" %5lu %5lu") %inode %pNode->getID() << endl;
				};

				// Face
				size_t nNFace=pSgrp->getNumOfFace();
				for(size_t iface=0; iface < nNFace; iface++){
				
					ofs << format(" %s %5lu %5lu %5lu %5lu") %pSgrp->getFaceType(iface) %iface %pSgrp->getElemID(iface) %pSgrp->getConvMW3FaceN(iface) %0;

					vector<size_t> vBNode = pSgrp->getBNodeN_Face(iface);
					for(size_t ibnode=0; ibnode < vBNode.size(); ibnode++) ofs << format(" %5lu") %vBNode[ibnode];
					ofs << format("   %10.3e") %0.000 << endl;//El
				};
				ofs << "End\n" << endl;
			}else{
				/*
				^OFBoundaryFace
				( fBN^ )
				ʋEbVID@E  bVp[cID  Eߓ_ Face
				Eߓ_ID  ߓ_ID  DOF@ DOFԍ El DOFԍ El DOFԍ Elc
				c
				ʌ`^Cv EID  vfID  ʔԍ  @ Eߓ_ID...(JԂ)  
				c
				*/
				ofs << "BoundaryFace" << endl;
				ofs << format(" %lu %s %5lu %5lu %5lu") %pSgrp->getID() %msDirichlet %pMesh->getID() %pSgrp->getNumOfNode() %pSgrp->getNumOfFace() << endl;
				// BNode 
				size_t nNNode=pSgrp->getNumOfNode();
				for(size_t inode=0; inode < nNNode; inode++){
					CNode* pNode = pSgrp->getNode(inode);
					ofs << format(" %5lu %5lu %lu %lu %lf") %inode %pNode->getID() %1 %0 %0.000 << endl;
				};
				// Face
				size_t nNFace=pSgrp->getNumOfFace();
				for(size_t iface=0; iface < nNFace; iface++){
				
					ofs << format(" %s %5lu %5lu %5lu") %pSgrp->getFaceType(iface) %iface %pSgrp->getElemID(iface) %pSgrp->getConvMW3FaceN(iface);

					vector<size_t> vBNode = pSgrp->getBNodeN_Face(iface);
					for(size_t ibnode=0; ibnode < vBNode.size(); ibnode++) ofs << format(" %5lu") %vBNode[ibnode];
					ofs << endl;
				};
				ofs << "End\n" << endl;
			}//if(BndType)
		};
	};


	// bZ[W\
	string sSumGrpN= lexical_cast<string>(nSumGrpN);
	pMsg->info("wrote :"+ sSumGrpN +" block \"BoundaryFace\"");

	return true;
}
//--
// BoundaryEdgeMesh : ftHgNeumann^ŏo(R:FrontISTR͋ElgpȂ)
//--
bool CFileWriter::WriteBndEdgeMesh(CAssyModel *pAssyModel, ofstream& ofs)
{
	if(!pAssyModel->existMesh()) return false;//Mesh

	CMessage *pMsg=CMessage::Instance();

	size_t nNMesh=pAssyModel->getNumOfMesh();
	size_t nSumGrpN=0;
	string sBndType;
	/*
	^OFBoundaryEdgeMesh
	
	bVp[cID  E
	ӋEbVID  Eށ@  Rx Rxԍ Rxԍ ...(JԂ) (DOF) DOF  DOF  ...
	c
	*/
	for(size_t i=0; i < nNMesh; i++){
		CMesh *pMesh=pAssyModel->getMesh(i);

		size_t nNLgrp=pMesh->getNumOfLgrp();
		nSumGrpN += nNLgrp;

		if(nNLgrp > 0){
			ofs << "BoundaryEdgeMesh" << endl;
			ofs << format(" %lu %lu") %pMesh->getID() %nNLgrp << endl;
			for(size_t ii=0; ii < nNLgrp; ii++){
				CLgroup* pLgrp=pMesh->getLgrp(ii);

				switch(pLgrp->getBndType()){ //EނH
				case( BndType::Dirichlet ): sBndType = msDirichlet; break;
				case( BndType::Neumann ): sBndType = msNeumann; break;
				case( BndType::NotUse ): sBndType = msNeumann; break;
				default: 
					sBndType = msNeumann;
					break;
				}

				ofs << format(" %lu %s %s %lu %lu %lu") %pLgrp->getID() %sBndType %pLgrp->getGroupName() %1 %0 %0 << endl;//Rx:1,Rx:0,:0
			};
			ofs << "End\n" << endl;
		}
	};


	for(size_t i=0; i < nNMesh; i++){
		CMesh *pMesh=pAssyModel->getMesh(i);

		size_t nNLgrp=pMesh->getNumOfLgrp();
		
		for(size_t ii=0; ii < nNLgrp; ii++){
			CLgroup *pLgrp= pMesh->getLgrp(ii);
			if(pLgrp->getBndType()!=BndType::Dirichlet){
				/*
				^OFBoundaryEdge
				 ( mC} )
				ӋEbVID@E  bVp[cID  Eߓ_@Edge
				Eߓ_ID  ߓ_ID 
				c
				ӌ`^Cv EID  vfID  Ӕԍ  Rxԍ Eߓ_ID Eߓ_ID El
				c
				*/
				ofs << "BoundaryEdge" << endl;
				ofs << format(" %lu %s %5lu %5lu %5lu") %pLgrp->getID() %msNeumann %pMesh->getID() %pLgrp->getNumOfNode() %pLgrp->getNumOfEdge() << endl;
				// BNode 
				size_t nNNode=pLgrp->getNumOfNode();
				for(size_t inode=0; inode < nNNode; inode++){
					CNode* pNode = pLgrp->getNode(inode);
					ofs << format(" %5lu %5lu") %inode %pNode->getID() << endl;
				};
				// Edge
				size_t nNEdge=pLgrp->getNumOfEdge();
				for(size_t iedge=0; iedge < nNEdge; iedge++){
					ofs << format(" %s %5lu %5lu %5lu %5lu") %pLgrp->getEdgeType(iedge) %iedge %pLgrp->getElemID(iedge) %pLgrp->getConvMW3EdgeN(iedge) %0;

					vector<size_t> vBNode = pLgrp->getBNode_Edge(iedge);
					for(size_t ibnode=0; ibnode < vBNode.size(); ibnode++) ofs << format(" %5lu") %vBNode[ibnode];
					ofs << format("   %10.3e") %0.000 << endl;//El
				};
				ofs << "End\n" << endl;
			}else{
				/*
				^OFBoundaryEdge
				 ( fBN )
				ӋEbVID@E  bVp[cID  Eߓ_@Edge
				Eߓ_ID  ߓ_ID  DOF@ DOFԍ El DOFԍ El DOFԍ Elc
				c
				ӌ`^Cv EID  vfID  Ӕԍ  Eߓ_ID Eߓ_ID 
				c
				*/
				ofs << "BoundaryEdge" << endl;
				ofs << format(" %lu %s %5lu %5lu %5lu") %pLgrp->getID() %msDirichlet %pMesh->getID() %pLgrp->getNumOfNode() %pLgrp->getNumOfEdge() << endl;
				// BNode 
				size_t nNNode=pLgrp->getNumOfNode();
				for(size_t inode=0; inode < nNNode; inode++){
					CNode* pNode = pLgrp->getNode(inode);
					ofs << format(" %5lu %5lu %lu %lu %lf") %inode %pNode->getID() %1 %0 %0.000 << endl;
				};
				// Edge
				size_t nNEdge=pLgrp->getNumOfEdge();
				for(size_t iedge=0; iedge < nNEdge; iedge++){
					ofs << format(" %s %5lu %5lu %5lu") %pLgrp->getEdgeType(iedge) %iedge %pLgrp->getElemID(iedge) %pLgrp->getConvMW3EdgeN(iedge);

					vector<size_t> vBNode = pLgrp->getBNode_Edge(iedge);
					for(size_t ibnode=0; ibnode < vBNode.size(); ibnode++) ofs << format(" %5lu") %vBNode[ibnode];
					ofs << endl;
				};
				ofs << "End\n" << endl;
			}//if(BndType)
		};
	};

	// bZ[W\
	string sSumGrpN= lexical_cast<string>(nSumGrpN);
	pMsg->info("wrote :"+ sSumGrpN +" block \"BoundaryEdge\"");

	return true;
}
//--
// BoundaryVolumeMesh : ftHgNeumann^ŏo(R:FrontISTR͋ElgpȂ)
//--
bool CFileWriter::WriteBndVolMesh(CAssyModel *pAssyModel, ofstream& ofs)
{
	if(!pAssyModel->existMesh()) return false;//Mesh

	CMessage* pMsg=CMessage::Instance();

	size_t nNMesh=pAssyModel->getNumOfMesh();
	size_t nSumGrpN=0;
	string sBndType;
	/*
	^OFBoundaryVolumeMesh
	
	bVp[cID  E
	̐ϋEbVID  Eށ@  Rx Rxԍ Rxԍ ...(JԂ) (DOF) DOF  DOF  ...
	c
	*/
	for(size_t i=0; i < nNMesh; i++){
		CMesh *pMesh=pAssyModel->getMesh(i);
		nSumGrpN += pMesh->getNumOfEgrp();

		if(pMesh->getNumOfEgrp() > 0){
			ofs << "BoundaryVolumeMesh" << endl;
			ofs << format(" %lu %lu") %pMesh->getID() %pMesh->getNumOfEgrp() << endl;
			for(size_t ii=0; ii < pMesh->getNumOfEgrp(); ii++){
				CEgroup *pEgrp=pMesh->getEgrp(ii);

				switch(pEgrp->getBndType()){ //EނH
				case( BndType::Dirichlet ): sBndType = msDirichlet; break;
				case( BndType::Neumann ): sBndType = msNeumann; break;
				case( BndType::NotUse ): sBndType = msNeumann; break;
				default: 
					sBndType = msNeumann;
					break;
				}

				ofs << format(" %lu %s %s %lu %lu %lu") %pEgrp->getID() %sBndType %pEgrp->getGroupName() %1 %0 %0 << endl;//Rx:1,Rx:0,:0
			};
			ofs << "End\n" << endl;
		}
	};

	// regex
	string strRegex= FistrElemTypeS::regStandard();// MW3ɑ݂vf^Cv
	sregex regBasis = sregex::compile(strRegex);
	string strRegex2= FistrElemTypeS::regShellType();// VFvf
	sregex regShell= sregex::compile(strRegex2);
	smatch res;

	for(size_t i=0; i < nNMesh; i++){
		CMesh *pMesh=pAssyModel->getMesh(i);
		size_t nNEgrp= pMesh->getNumOfEgrp();
		for(size_t ii=0; ii < nNEgrp; ii++){
			CEgroup *pEgrp=pMesh->getEgrp(ii);

			if(pEgrp->getBndType()!=BndType::Dirichlet){
				/*
				^OFBoundaryVolume
				 ( mC} )
				̐ϋEbVID@E  bVp[cID  Eߓ_ vf
				Eߓ_ID  ߓ_ID  
				c
				`^Cv   E̐ID  vfID  0  Rxԍ Eߓ_ID...(JԂ)   El
				c
				*/
				ofs << "BoundaryVolume" << endl;
				ofs << format(" %lu %s %5lu %5lu %5lu") %pEgrp->getID() %msNeumann %pMesh->getID() %pEgrp->getNumOfNode() %pEgrp->getNumOfElement() << endl;
				// Node
				for(size_t inode=0; inode < pEgrp->getNumOfNode(); inode++){
					ofs << format(" %5lu %5lu") %inode %pEgrp->getNodeID(inode) << endl;
				};
				// Volume(Element)
				for(size_t ielem=0; ielem < pEgrp->getNumOfElement(); ielem++){

					CElement *pElem= pEgrp->getElement(ielem);
					string sType= pElem->getCType();
				
					if( regex_search(sType,res,regBasis) ){
						// vf`MWRɑ.
						ofs << format(" %s %5lu %5lu %5lu %5lu") %sType %ielem %pElem->getID() %0 %0 ;
					}else if( regex_search(sType,res,regShell) ){
						// VFvf˒u
						if(sType==FistrElemTypeS::TriShell()) sType=FistrElemTypeS::Triangle();
						if(sType==FistrElemTypeS::TriShell2())sType=FistrElemTypeS::Triangle2();
						if(sType==FistrElemTypeS::QuadShell())sType=FistrElemTypeS::Quad();
						if(sType==FistrElemTypeS::QuadShell2())sType=FistrElemTypeS::Quad2();
						ofs << format(" %s %5lu %5lu %5lu %5lu") %sType %ielem %pElem->getID() %0 %0 ;
					}else{
						// KiO
						pMsg->error( res.str()+" not supported");//Error --- IFaceQuad܂.
					}
					// vf\BNode
					for(size_t enode=0; enode < pElem->getNumOfNode(); enode++){
						size_t nID= pElem->getNodeID_Fistr2MW3(enode);
						size_t nIX= pEgrp->getBNodeIndex(nID);
						ofs << format(" %5lu") %nIX;
					};
					ofs << format("   %10.3e") %0.000 << endl;//El
				};
				ofs << "End\n" << endl;
			}else{
				/*
				^OFBoundaryVolume
				 ( fBN )
				̐ϋEbVID@E  bVp[cID  Eߓ_ vf
				Eߓ_ID  ߓ_ID  DOF@ DOFԍ El DOFԍ El DOFԍ Elc
				c
				`^Cv   E̐ID  vfID  0  @Eߓ_ID...(JԂ)   
				c
				*/
				ofs << "BoundaryVolume" << endl;
				ofs << format(" %lu %s %5lu %5lu %5lu") %pEgrp->getID() %msDirichlet %pMesh->getID() %pEgrp->getNumOfNode() %pEgrp->getNumOfElement() << endl;
				// Node
				for(size_t inode=0; inode < pEgrp->getNumOfNode(); inode++){
					ofs << format(" %5lu %5lu %lu %lu %lf") %inode %pEgrp->getNodeID(inode) %1 %0 %0.000 << endl;
				};
				// Volume(Element)
				for(size_t ielem=0; ielem < pEgrp->getNumOfElement(); ielem++){

					CElement *pElem= pEgrp->getElement(ielem);
					string sType= pElem->getCType();
				
					if( regex_search(sType,res,regBasis) ){
						// vf`MWRɑ.
						ofs << format(" %s %5lu %5lu %5lu") %sType %ielem %pElem->getID() %0 ;
					}else if( regex_search(sType,res,regShell) ){
						// VFvf˒u
						if(sType==FistrElemTypeS::TriShell()) sType=FistrElemTypeS::Triangle();
						if(sType==FistrElemTypeS::TriShell2())sType=FistrElemTypeS::Triangle2();
						if(sType==FistrElemTypeS::QuadShell())sType=FistrElemTypeS::Quad();
						if(sType==FistrElemTypeS::QuadShell2())sType=FistrElemTypeS::Quad2();
						ofs << format(" %s %5lu %5lu %5lu") %sType %ielem %pElem->getID() %0;
					}else{
						// KiO
						pMsg->error( res.str()+" not supported");//Error --- IFaceQuad܂.
					}
					// vf\BNode
					for(size_t enode=0; enode < pElem->getNumOfNode(); enode++){
						size_t nID= pElem->getNodeID_Fistr2MW3(enode);
						size_t nIX= pEgrp->getBNodeIndex(nID);
						ofs << format(" %5lu") %nIX;
					};
					ofs << endl;
				};
				ofs << "End\n" << endl;
			}//if(BndType)
		};
	};


	// bZ[W\
	string sSumGrpN= lexical_cast<string>(nSumGrpN);
	pMsg->info("wrote :"+ sSumGrpN +" block \"BoundaryVolume\"");

	return true;
}
//--
// ElementGroup
//    BoundaryVolumeMeshf[^ElementGroupƂēǂݑւďo
//--
bool CFileWriter::WriteElemGroup(CAssyModel *pAssyModel, ofstream& ofs)
{
	if(!pAssyModel->existMesh()) return false;//Mesh

	CMessage* pMsg=CMessage::Instance();

	size_t nNMesh=pAssyModel->getNumOfMesh();
	size_t nSumGrpN=0;

	/*
	ElementGroup
		O[vԍ  O[v  bVp[cԍ
		c
	End
	*/
	for(size_t i=0; i < nNMesh; i++){
		CMesh *pMesh=pAssyModel->getMesh(i);
		nSumGrpN += pMesh->getNumOfEgrp();
	};
	if(nSumGrpN > 0){
		ofs << "ElementGroup" << endl;
		for(size_t i=0; i < nNMesh; i++){
			CMesh *pMesh=pAssyModel->getMesh(i);
			for(size_t ii=0; ii < pMesh->getNumOfEgrp(); ii++){
				CEgroup *pEgrp=pMesh->getEgrp(ii);
				ofs << format(" %lu %s %lu") %pEgrp->getID() %pEgrp->getGroupName() %pMesh->getID() << endl;
			};//---for(O[v)
		};//---for(bV)end
		ofs << "End\n" << endl;
	}

	/*
	ElementGroupEntity
		O[vԍ  bVp[cԍ
		vfID  vfID  vfID  vfID  vfID
		vfID  vfID  vfID  vfID  vfID
		c
	End
	*/
	if(nSumGrpN > 0){
		for(size_t i=0; i < nNMesh; i++){
			CMesh *pMesh=pAssyModel->getMesh(i);
			for(size_t ii=0; ii < pMesh->getNumOfEgrp(); ii++){
				CEgroup *pEgrp=pMesh->getEgrp(ii);
				ofs << "ElementGroupEntity" << endl;
				ofs << format(" %lu %lu")  %pEgrp->getID() %pMesh->getID() << endl;

				for(size_t ielem=0; ielem < pEgrp->getNumOfElement(); ielem++){
					CElement *pElem= pEgrp->getElement(ielem);
					
					ofs << format(" %lu")  %pElem->getID();
					if(ielem!=0 && ielem%5==0) ofs << endl;
				};
				if((pEgrp->getNumOfElement()-1)%5!=0) ofs << endl;
				ofs << "End\n" << endl;
			};
		};
	}


	// bZ[W\
	string sSumGrpN= lexical_cast<string>(nSumGrpN);
	pMsg->info("wrote :"+ sSumGrpN +" block \"ElementGroup\"");

	return true;
}

//--
// ContactMesh
//--
bool CFileWriter::WriteContactMesh(CAssyModel *pAssyModel, ofstream& ofs)
{
	if(!pAssyModel->existConPair() && !pAssyModel->existMPCPair()) return false;//MPCContact̗Ƃ

	CMessage *pMsg=CMessage::Instance();
	
	/*
	^OFContactMesh
	
	ڍbV őID  ŏID
	ڍbVID  gRank @ԍ(0:MPC, 1:Contact, ...)
	ڍߓ_ őID  ŏID
	ڍߓ_ID  X  Y  Z  bVp[cID  ߓ_ID  rank  maslave  v 3 0 ("v 3 0"̓_~[萔)
	c
	}X^[ڍʐ  őID  ŏID
	}X^[ID bVp[cID  vfID  (vf)ʔԍ  `  ڍߓ_ID c  gRank
	c
	X[uڍʐ  őID  ŏID
	X[uID bVp[cID  vfID  (vf)ʔԍ  `  ڍߓ_ID c  gRank
	c
	
	 maslaveF0:}X^[A1:X[u
	*/

	size_t nNumMPC= pAssyModel->getNumOfMPCPair();
	
	//MPCS 
	ofs << "ContactMesh" << endl;
	ofs << format(" %lu %lu %lu") %nNumMPC %pAssyModel->getMaxIDinMPCPair() %pAssyModel->getMinIDinMPCPair() << endl;
	
	for(size_t i=0; i < nNumMPC; i++){
		// MPCڍ(P-ڍ)
		ofs << format(" %lu %lu %lu") %i %0 %0 << endl;
		
		CGroupPair *pMPCPair= pAssyModel->getMPCPair(i);
		
		size_t nAllNNum= pMPCPair->getNumOfAllNode();   //ߓ_
		size_t nMNNum  = pMPCPair->getNumOfMasterNode();//}X^[ߓ_
		size_t nMMeshID= pMPCPair->getMasterMeshID();   //}X^[p[cID
		size_t nSMeshID= pMPCPair->getSlaveMeshID();    //X[up[cID
		// ڍߓ_
		ofs << format(" %9lu %9lu %9lu") %nAllNNum %pMPCPair->getMaxNodeIX() %pMPCPair->getMinNodeIX() << endl;
		for(size_t inode=0; inode < nAllNNum; inode++){

			CNode* pNode= pMPCPair->getNode(inode);
			double X=pNode->getX(), Y=pNode->getY(), Z=pNode->getZ();
			size_t nNID=pNode->getID();
			if(inode < nMNNum){
				//}X^[_
				ofs << format(" %9lu %23.15e %23.15e %23.15e %lu %9lu %lu %lu %s") %inode %X %Y %Z %nMMeshID %nNID %0 %0 %"v 3 0" << endl;
			}else{
				//X[u_
				ofs << format(" %9lu %23.15e %23.15e %23.15e %lu %9lu %lu %lu %s") %inode %X %Y %Z %nSMeshID %nNID %0 %1 %"v 3 0" << endl;
			}
		};

		//}X^[
		size_t nMFaceNum= pMPCPair->getNumOfMasterFace();
		size_t nMaxMFaceIX = pMPCPair->getMaxMFaceIX();
		size_t nMinMFaceIX = pMPCPair->getMinMFaceIX();
		ofs << format(" %9lu %9lu %9lu") %nMFaceNum %nMaxMFaceIX %nMinMFaceIX << endl;
		// }X^[ID bVp[cID  vfID  (vf)ʔԍ  `  ڍߓ_ID c  gRank
		for(size_t iface=0; iface < nMFaceNum; iface++){
			size_t nElemID= pMPCPair->getMasterElemID(iface);
			size_t nFaceN = pMPCPair->getMasterFaceN_MW3(iface);
			string sFaceTye=pMPCPair->getMasterFaceType(iface);
			ofs << format(" %9lu %9lu %9lu %9lu %s") %iface %nMMeshID %nElemID %nFaceN %sFaceTye;

			vector<size_t> vConIX= pMPCPair->getMFaceNodeIX(iface);
			for(size_t inode=0; inode < vConIX.size(); inode++){
				ofs << format(" %9lu") %vConIX[inode];
			};
			ofs << format(" %lu") %0 << endl;
		};

		//X[u
		size_t nSFaceNum= pMPCPair->getNumOfSlaveFace();
		size_t nMaxSFaceIX= pMPCPair->getMaxSFaceIX();
		size_t nMinSFaceIX= pMPCPair->getMinSFaceIX();
		ofs << format(" %9lu %9lu %9lu") %nSFaceNum %nMaxSFaceIX %nMinSFaceIX << endl;
		// X[uID bVp[cID  vfID  (vf)ʔԍ  `  ڍߓ_ID c  gRank
		for(size_t iface=0; iface < nSFaceNum; iface++){
			size_t nElemID= pMPCPair->getSlaveElemID(iface);
			size_t nFaceN = pMPCPair->getSlaveFaceN_MW3(iface);
			string sFaceTye=pMPCPair->getSlaveFaceType(iface);
			ofs << format(" %9lu %9lu %9lu %9lu %s") %iface %nSMeshID %nElemID %nFaceN %sFaceTye;

			vector<size_t> vConIX= pMPCPair->getSFaceNodeIX(iface);
			for(size_t inode=0; inode < vConIX.size(); inode++){
				ofs << format(" %9lu") %vConIX[inode];
			};
			ofs << format(" %lu") %0 << endl;
		};

	};
	ofs << "End\n" << endl;

	string sNumMPC= lexical_cast<string>(nNumMPC);
	pMsg->info("wrote :"+sNumMPC+" block \"ContactMesh(MPC)\"" );
	
	return true;
}


//--
// MW3 bVo
//--
void CFileWriter::WriteMesh_MW3(CAssyModel *pAssyModel, string filename)
{
	CMessage *pMsg=CMessage::Instance();

	string strMW3std  = FileName::regMW3Mesh();
	string strMW3fistr= FileName::regMW3FistrMesh();
	sregex regMW3std  = sregex::compile(strMW3std);
	sregex regMW3fistr= sregex::compile(strMW3fistr);
	smatch res1, res2;
	//--
	// t@C`FbN : rankԍ"0"̕t
	//--
	// 1.MW3W          bVt@C:".0.msh"
	// 2.MW3FrontISTRΉ bVt@C:".msh.0"
	//
	if(!regex_search(filename, res1, regMW3std) && !regex_search(filename, res2, regMW3fistr) ){
		// Warn
		pMsg->warn("invalid output file extension.");

		string user_str;
		pMsg->user_in("select file extension kind: enter STD or FISTR => ", user_str);//----------------------- [U[I
		transform(user_str.begin(), user_str.end(), user_str.begin(), ::tolower);//-- Sďɕϊ 

		string strExt = FileName::regExt();
		sregex regExt = sregex::compile(strExt);
		smatch resExt;
		
		if( regex_search(filename, resExt, regExt) ){
			//--
			// gq".msh"tĂ邪rankԍ͖ꍇ
			//--
			if(user_str=="std"){//MW3W
				filename = regex_replace(filename, regExt, string(".0.msh") );//".msh"".0.msh"ɒu
			}else if(user_str=="fistr"){//MW3FrontISTRΉ
				filename = regex_replace(filename, regExt, string(".msh.0") );//".msh"".msh.0"ɒu
			}else{
				pMsg->error("input error, change to std type.");
				filename = regex_replace(filename, regExt, string(".0.msh") );//".msh"".0.msh"ɒu
			}
		}else{
			//--
			// gq.mshȂꍇ
			//--
			if(user_str=="std"){   filename += ".0.msh";}else
			if(user_str=="fistr"){ filename += ".msh.0";}
			else{ pMsg->error("input error, change to std type."); filename += ".0.msh";}
		}
	}//if(gqrankԍ̗L) 


	ofstream ofs;
	ofs.open(filename.c_str());// Open
	//--
	// t@Co
	//--
	if(ofs){
		pMsg->info("file open:\""+filename+"\"");

		WriteHeader(ofs);
		WriteAssyModel(pAssyModel, ofs);//---AssyModel
		WriteNode(pAssyModel, ofs);     //---Node
		WriteElement(pAssyModel, ofs);  //---Element
		WriteBndNodeMesh(pAssyModel, ofs);//-BoundaryNodeMesh
		WriteBndFaceMesh(pAssyModel, ofs);//-BoundaryFaceMesh
		WriteBndEdgeMesh(pAssyModel, ofs);//-BoundaryEdgeMesh
		////WriteBndVolMesh(pAssyModel, ofs); //-BoundaryVolumeMesh  fistr v4 ͕sgp
		WriteElemGroup(pAssyModel, ofs);//---ElementGroup  BoundaryVolumeMesh̓ǂݑւ
		//
		// # ʐMEʂ̓VOȂ̂Ŗ : CommMesh2
		//
		WriteContactMesh(pAssyModel, ofs);//-ContactMesh

		pMsg->info("file close:\""+filename+"\"\n");
	}else{
		pMsg->error("\""+filename+"\" could not open.");
		exit(0);//------------------------------------------ exit(0)
	}

	ofs.close();// Close
}



